@@ -10,7 +10,7 @@ import string
 import time
 from abc import abstractmethod
 from functools import partial, wraps
-from typing import Union
+from typing import cast, Union
 
 import requests
 import yaml
@@ -19,7 +19,7 @@ from selenium.webdriver.remote.webdriver import WebDriver
 
 from galaxy.util import DEFAULT_SOCKET_TIMEOUT
 from . import sizzle
-from .components import Component
+from .components import Component, HasText
 from .data import (
     load_root_component,
 )
@@ -27,7 +27,7 @@ from .has_driver import (
     exception_indicates_not_clickable,
     exception_indicates_stale_element,
     HasDriver,
-    TimeoutException,
+    SeleniumTimeoutException,
 )
 from .smart_components import SmartComponent
 
@@ -205,7 +205,7 @@ class NavigatesGalaxy(HasDriver):
         """
         time.sleep(duration)
 
-    def timeout_for(self, wait_type: WaitType = DEFAULT_WAIT_TYPE) -> float:
+    def timeout_for(self, wait_type: WaitType = DEFAULT_WAIT_TYPE, **kwd) -> float:
         return self.wait_length(wait_type)
 
     def home(self) -> None:
@@ -365,12 +365,12 @@ class NavigatesGalaxy(HasDriver):
 
         def history_has_hid(driver):
             hids = get_hids()
-            return any([h == hid for h in hids])
+            return any(h == hid for h in hids)
 
         timeout = self.timeout_for(wait_type=WAIT_TYPES.JOB_COMPLETION)
         try:
             self.wait(timeout).until(history_has_hid)
-        except self.TimeoutException as e:
+        except SeleniumTimeoutException as e:
             hids = get_hids()
             message = f"Timeout waiting for history {history_id} to have hid {hid} - have hids {hids}"
             raise self.prepend_timeout_message(e, message)
@@ -385,7 +385,7 @@ class NavigatesGalaxy(HasDriver):
         history_item_selector = self.history_panel_item_component(history_item, hid=hid, multi_history_panel=multi_history_panel)
         try:
             self.history_item_wait_for(history_item_selector, allowed_force_refreshes)
-        except self.TimeoutException as e:
+        except SeleniumTimeoutException as e:
             contents_elements = self.find_elements(self.navigation.history_panel.selectors.contents)
             div_ids = [f"#{d.get_attribute('id')}" for d in contents_elements]
             template = "Failed waiting on history item %d to become visible, visible datasets include [%s]."
@@ -406,7 +406,7 @@ class NavigatesGalaxy(HasDriver):
             try:
                 rval = self.wait_for_visible(history_item_selector, wait_type=WAIT_TYPES.JOB_COMPLETION)
                 break
-            except self.TimeoutException:
+            except SeleniumTimeoutException:
                 if attempt >= allowed_force_refreshes:
                     raise
 
@@ -435,7 +435,7 @@ class NavigatesGalaxy(HasDriver):
             history_item_selector_state = history_item_selector.with_data("state", state)
         try:
             self.history_item_wait_for(history_item_selector_state, allowed_force_refreshes)
-        except self.TimeoutException as e:
+        except SeleniumTimeoutException as e:
             history_item = self.wait_for_visible(history_item_selector)
             current_state = "UNKNOWN"
             classes = history_item.get_attribute("class").split(" ")
@@ -576,15 +576,15 @@ class NavigatesGalaxy(HasDriver):
             # Make sure the user menu was dropped down
             user_menu = self.components.masthead.user_menu.wait_for_visible()
             try:
-                user_email_element = self.components.masthead.user_email.wait_for_visible()
-            except self.TimeoutException as e:
+                username_element = self.components.masthead.username.wait_for_visible()
+            except SeleniumTimeoutException as e:
                 menu_items = user_menu.find_elements_by_css_selector("li a")
                 menu_text = [mi.text for mi in menu_items]
                 message = f"Failed to find logged in message in menu items {', '.join(menu_text)}"
                 raise self.prepend_timeout_message(e, message)
 
-            text = user_email_element.text
-            assert email in text
+            text = username_element.text
+            assert username in text
             assert self.get_logged_in_user()["email"] == email
 
             # clicking away no longer closes menu post Masthead -> VueJS
@@ -593,7 +593,7 @@ class NavigatesGalaxy(HasDriver):
     def wait_for_logged_in(self):
         try:
             self.components.masthead.logged_in_only.wait_for_visible()
-        except self.TimeoutException as e:
+        except SeleniumTimeoutException as e:
             ui_logged_out = self.components.masthead.logged_out_only.is_displayed
             if ui_logged_out:
                 dom_message = "Element a.loggedout-only is present in DOM, indicating Login or Register button still in masthead."
@@ -727,7 +727,7 @@ class NavigatesGalaxy(HasDriver):
         self.wait_for_and_click_selector(build_selector)
         try:
             self.wait_for_selector_absent_or_hidden(build_selector)
-        except TimeoutException:
+        except SeleniumTimeoutException:
             # Sometimes the callback in the JS hasn't be registered by the
             # time that the build button is clicked. By the time the timeout
             # has been registered - it should have been.
@@ -1061,7 +1061,6 @@ class NavigatesGalaxy(HasDriver):
         search_box.send_keys(text)
         value = search_box.get_attribute("value")
         assert value == text, value
-        self.driver.execute_script("$(arguments[0]).keyup();", search_box)
 
     def libraries_folder_create(self, name):
         self.components.libraries.folder.add_folder.wait_for_and_click()
@@ -1245,19 +1244,25 @@ class NavigatesGalaxy(HasDriver):
         self.driver.execute_script("arguments[0].scrollIntoView(true);", tool_element)
         tool_link.wait_for_and_click()
 
+    def datasource_tool_open(self, tool_id):
+        tool_link = self.components.tool_panel.data_source_tool_link(tool_id=tool_id)
+        tool_element = tool_link.wait_for_present()
+        self.driver.execute_script("arguments[0].scrollIntoView(true);", tool_element)
+        tool_link.wait_for_and_click()
+
     def tool_parameter_div(self, expanded_parameter_id):
         return self.components.tool_form.parameter_div(parameter=expanded_parameter_id).wait_for_clickable()
 
-    def tool_parameter_edit_rules(self, expanded_parameter_id="rules"):
+    def tool_parameter_edit_rules(self):
         rules_div_element = self.tool_parameter_div("rules")
         edit_button_element = rules_div_element.find_element_by_css_selector("i.fa-edit")
         edit_button_element.click()
 
-    def tool_set_value(self, expanded_parameter_id, value, expected_type=None, test_data_resolver=None):
+    def tool_set_value(self, expanded_parameter_id, value, expected_type=None):
         div_element = self.tool_parameter_div(expanded_parameter_id)
         assert div_element
         if expected_type in ["select", "data", "data_collection"]:
-            div_selector = f"div.ui-form-element[tour_id$='{expanded_parameter_id}']"
+            div_selector = f"div.ui-form-element[id$='form-element-{expanded_parameter_id}']"
             self.select2_set_value(div_selector, value)
         else:
             input_element = div_element.find_element_by_css_selector("input")
@@ -1340,7 +1345,7 @@ class NavigatesGalaxy(HasDriver):
 
     def history_panel_click_copy_elements(self):
         if self.is_beta_history():
-            self.use_bootstrap_dropdown(option="copy datasets", menu="new content menu")
+            self.use_bootstrap_dropdown(option="copy datasets", menu="history action menu")
         else:
             self.click_history_option("Copy Datasets")
 
@@ -1461,6 +1466,17 @@ class NavigatesGalaxy(HasDriver):
         dataset_selector = next_level_element_selector.descendant(".dataset")
         self.wait_for_and_click(dataset_selector)
 
+    def history_panel_item_view_dataset_details(self, hid):
+        if not self.is_beta_history():
+            self.history_panel_ensure_showing_item_details(hid)
+            self.hda_click_details(hid)
+            self.components.dataset_details._.wait_for_visible()
+        else:
+            item = self.history_panel_item_component(hid=hid)
+            item.dataset_operations_dropdown.wait_for_and_click()
+            item.info_button.wait_for_and_click()
+            self.components.dataset_details._.wait_for_visible()
+
     def history_panel_item_click_visualization_menu(self, hid):
         viz_button_selector = f"{self.history_panel_item_selector(hid)} .visualizations-dropdown"
         self.wait_for_and_click_selector(viz_button_selector)
@@ -1511,12 +1527,15 @@ class NavigatesGalaxy(HasDriver):
             details_component.wait_for_visible()
         return details_component
 
-    def hda_click_primary_action_button(self, hid, button_key):
-        item_component = self.history_panel_click_item_title(hid=hid, wait=True)
-        item_component.primary_action_buttons.wait_for_visible()
+    def hda_click_primary_action_button(self, hid: int, button_key: str):
+        self.history_panel_ensure_showing_item_details(hid)
+        item_component = self.history_panel_item_component(hid=hid)
         button_component = item_component[f"{button_key}_button"]
         button_component.wait_for_and_click()
 
+    def hda_click_details(self, hid: int):
+        self.hda_click_primary_action_button(hid, "info")
+
     def history_panel_click_item_title(self, hid, **kwds):
         item_component = self.history_panel_item_component(hid=hid)
         details_component = item_component.details
@@ -1571,7 +1590,7 @@ class NavigatesGalaxy(HasDriver):
         self.components.masthead.logout.wait_for_and_click()
         try:
             self.components.masthead.logged_out_only.wait_for_visible()
-        except self.TimeoutException as e:
+        except SeleniumTimeoutException as e:
             message = "Clicked logout button but waiting for 'Login or Registration' button failed, perhaps the logout button was clicked before the handler was setup?"
             raise self.prepend_timeout_message(e, message)
         assert not self.is_logged_in(), "Clicked to logged out and UI reflects a logout, but API still thinks a user is logged in."
@@ -1660,12 +1679,18 @@ class NavigatesGalaxy(HasDriver):
         """
         return self.assert_absent_or_hidden(selector)
 
-    def assert_tooltip_text(self, element, expected, sleep=0, click_away=True):
+    def assert_tooltip_text(self, element, expected: Union[str, HasText], sleep: int = 0, click_away: bool = True):
         if hasattr(expected, "text"):
-            expected = expected.text
+            expected = cast(HasText, expected).text
         text = self.get_tooltip_text(element, sleep=sleep, click_away=click_away)
         assert text == expected, f"Tooltip text [{text}] was not expected text [{expected}]."
 
+    def assert_tooltip_text_contains(self, element, expected: Union[str, HasText], sleep: int = 0, click_away: bool = True):
+        if hasattr(expected, "text"):
+            expected = cast(HasText, expected).text
+        text = self.get_tooltip_text(element, sleep=sleep, click_away=click_away)
+        assert expected in text, f"Tooltip text [{text}] was not expected text [{expected}]."
+
     def assert_error_message(self, contains=None):
         self.components._.messages.error.wait_for_visible()
         elements = self.find_elements(self.components._.messages.selectors.error)
@@ -1679,7 +1704,7 @@ class NavigatesGalaxy(HasDriver):
         if contains is not None:
 
             if type(element) == list:
-                assert any([contains in el.text for el in element]), \
+                assert any(contains in el.text for el in element), \
                     f"{contains} was not found in {[el.text for el in element]}"
                 return
 
@@ -1821,7 +1846,7 @@ class NavigatesGalaxy(HasDriver):
             self.assert_absent_or_hidden(editor)
 
 
-class NotLoggedInException(TimeoutException):
+class NotLoggedInException(SeleniumTimeoutException):
 
     def __init__(self, timeout_exception, user_info, dom_message):
         template = "Waiting for UI to reflect user logged in but it did not occur. API indicates no user is currently logged in. %s API response was [%s]. %s"
