@@ -75,6 +75,10 @@ class SmartTarget:
     def is_displayed(self):
         return self._has_driver.is_displayed(self._target)
 
+    @property
+    def is_absent(self):
+        return self._has_driver.element_absent(self._target)
+
     def wait_for_absent_or_hidden(self, **kwds):
         self._has_driver.wait_for_absent_or_hidden(self._target, **kwds)
 
@@ -96,5 +100,5 @@ class SmartTarget:
     def has_class(self, class_name):
         return class_name in self._has_driver.driver.find_element(*self._target.element_locator).get_attribute("class")
 
-    def wait_for_and_send_keys(self, text):
-        self.wait_for_visible().send_keys(text)
+    def wait_for_and_send_keys(self, *text):
+        self.wait_for_visible().send_keys(*text)
